@@ -52,6 +52,8 @@ class Route(object):
         self.routepath = routepath
         self.sub_domains = False
         self.prior = None
+        self.encoding = kargs.pop('_encoding', 'utf-8')
+        self.decode_errors = 'replace'
         
         # Don't bother forming stuff we don't need if its a static route
         self.static = kargs.get('_static', False)
@@ -106,6 +108,15 @@ class Route(object):
         self.hardcoded = frozenset([key for key in self.maxkeys \
             if key not in routekeys and self.defaults[key] is not None])
     
+    def make_unicode(self, s):
+        """Transform the given argument into a unicode string."""
+        if isinstance(s, unicode):
+            return s
+        elif isinstance(s, str):
+            return s.decode(self.encoding)
+        else:
+            return unicode(s)
+    
     def _pathkeys(self, routepath):
         """Utility function to walk the route, and pull out the valid 
         dynamic/wildcard keys."""
@@ -129,7 +140,7 @@ class Route(object):
                     done_on = ')'
                 else:
                     current = char
-                    done_on = self.done_chars
+                    done_on = self.done_chars + ('-',)
             elif collecting and char not in done_on:
                 current += char
             elif collecting:
@@ -194,8 +205,8 @@ class Route(object):
         defaultkeys = frozenset([key for key in kargs.keys() \
                                  if key not in reserved_keys])
         for key in defaultkeys:
-            if kargs[key] != None:
-                defaults[key] = unicode(kargs[key])
+            if kargs[key] is not None:
+                defaults[key] = self.make_unicode(kargs[key])
             else:
                 defaults[key] = None
         if 'action' in routekeys and not defaults.has_key('action') \
@@ -206,6 +217,7 @@ class Route(object):
             defaults['id'] = None
         newdefaultkeys = frozenset([key for key in defaults.keys() \
                                     if key not in reserved_keys])
+        
         return (defaults, newdefaultkeys)
         
     def makeregexp(self, clist):
@@ -263,7 +275,19 @@ class Route(object):
             elif self.prior in ['/', '#']:
                 partreg = '(?P<' + var + '>[^' + self.prior + ']+?)'
             else:
-                partreg = '(?P<' + var + '>[^%s]+?)' % ''.join(self.done_chars)
+                if not rest:
+                    partreg = '(?P<' + var + '>[^%s]+?)' % '/'
+                else:
+                    end = ''.join(self.done_chars)
+                    rem = rest
+                    if rem[0] == '\\' and len(rem) > 1:
+                        rem = rem[1]
+                    elif rem.startswith('(\\') and len(rem) > 2:
+                        rem = rem[2]
+                    else:
+                        rem = end
+                    rem = frozenset(rem) | frozenset(['/'])
+                    partreg = '(?P<' + var + '>[^%s]+?)' % ''.join(rem)
             
             if self.reqs.has_key(var):
                 noreqs = False
@@ -405,12 +429,13 @@ class Route(object):
         result = {}
         extras = frozenset(self.defaults.keys()) - frozenset(matchdict.keys())
         for key, val in matchdict.iteritems():
-            # change back into python unicode objects from the URL 
-            # representation
-            try:
-                val = val and urllib.unquote_plus(val).decode('utf-8', 'replace')
-            except:
-                val = val and urllib.unquote_plus(val).decode('utf-8', 'ignore')
+            if key != 'path_info' and self.encoding:
+                # change back into python unicode objects from the URL 
+                # representation
+                try:
+                    val = val and val.decode(self.encoding, self.decode_errors)
+                except UnicodeDecodeError:
+                    return False
             
             if not val and self.defaults.has_key(key) and self.defaults[key]:
                 result[key] = self.defaults[key]
@@ -442,7 +467,7 @@ class Route(object):
         if not _ignore_req_list:
             for key in self.reqs.keys():
                 val = kargs.get(key)
-                if val and not self.req_regs[key].match(unicode(val)):
+                if val and not self.req_regs[key].match(self.make_unicode(val)):
                     return False
         
         # Verify that if we have a method arg, its in the method accept list. 
@@ -473,8 +498,8 @@ class Route(object):
                     
                 # Now check to see if there's a default and it matches the 
                 # incoming call arg
-                if (has_default and has_arg) and unicode(kargs[arg]) == \
-                    unicode(self.defaults[arg]) and not gaps: 
+                if (has_default and has_arg) and self.make_unicode(kargs[arg]) == \
+                    self.make_unicode(self.defaults[arg]) and not gaps: 
                     continue
                 
                 # We need to pull the value to append, if the arg is None and 
@@ -493,7 +518,7 @@ class Route(object):
                 else:
                     return False
                 
-                urllist.append(url_quote(val))
+                urllist.append(url_quote(val, self.encoding))
                 if has_arg:
                     del kargs[arg]
                 gaps = True
@@ -501,7 +526,7 @@ class Route(object):
                 arg = part['name']
                 kar = kargs.get(arg)
                 if kar is not None:
-                    urllist.append(url_quote(kar))
+                    urllist.append(url_quote(kar, self.encoding))
                     gaps = True
             elif part and part[-1] in self.done_chars:
                 if not gaps and part in self.done_chars:
@@ -524,9 +549,18 @@ class Route(object):
             if _append_slash and not url.endswith('/'):
                 url += '/'
             url += '?'
-            url += urllib.urlencode([(key, kargs[key]) for key in kargs \
-                                     if key in extras and \
-                                     (key != 'action' or key != 'controller')])
+            fragments = []
+            for key in extras:
+                if key == 'action' or key == 'controller':
+                    continue
+                val = kargs[key]
+                if isinstance(val, (tuple, list)):
+                    for value in val:
+                        fragments.append((key, value))
+                else:
+                    fragments.append((key, val))
+                
+            url += urllib.urlencode(fragments)
         elif _append_slash and not url.endswith('/'):
             url += '/'
         return url
@@ -540,25 +574,24 @@ class Mapper(object):
     
     URL generation is done by passing keyword parameters into the generate function, a URL is then
     returned.
-        
     """
     def __init__(self, controller_scan=controller_scan, directory=None, 
                  always_scan=False, register=True, explicit=False):
         """Create a new Mapper instance
         
         All keyword arguments are optional.
-
+        
         ``controller_scan``
             Function reference that will be used to return a list of valid 
             controllers used during URL matching. If ``directory`` keyword arg
             is present, it will be passed into the function during its call. 
             This option defaults to a function that will scan a directory for
             controllers.
-
+        
         ``directory``
             Passed into controller_scan for the directory to scan. It should be
             an absolute path if using the default ``controller_scan`` function.
-
+        
         ``always_scan``
             Whether or not the ``controller_scan`` function should be run 
             during every URL match. This is typically a good idea during 
@@ -573,10 +606,29 @@ class Mapper(object):
         
         ``explicit``
             Boolean used to determine if routes should be connected with 
-            implicit defaults of:
-              {'controller':'content','action':'index','id':None}
+            implicit defaults of::
+                
+                {'controller':'content','action':'index','id':None}
+            
             When set to True, these defaults will not be added to route
             connections and ``url_for`` will not use Route memory.
+        
+        Additional attributes that may be set after mapper initialization (ie,
+        map.ATTRIBUTE = 'something'):
+        
+        ``encoding``
+            Used to indicate alternative encoding/decoding systems to use with
+            both incoming URL's, and during Route generation when passed a 
+            Unicode string. Defaults to 'utf-8'.
+        
+        ``decode_errors``
+            How to handle errors in the encoding, generally ignoring any chars 
+            that don't convert should be sufficient. Defaults to 'ignore'.
+        
+        ``hardcode_names``
+            Whether or not Named Routes result in the default options for the 
+            route being used *or* if they actually force url generation to use
+            the route. Defaults to False.
         """
         self.matchlist = []
         self.maxkeys = {}
@@ -597,6 +649,9 @@ class Mapper(object):
         self.sub_domains_ignore = []
         self.domain_match = '[^\.\/]+?\.[^\.\/]+'
         self.explicit = explicit
+        self.encoding = 'utf-8'
+        self.decode_errors = 'ignore'
+        self.hardcode_names = False
         if register:
             config = request_config()
             config.mapper = self
@@ -633,6 +688,14 @@ class Mapper(object):
         if '_explicit' not in kargs:
             kargs['_explicit'] = self.explicit
         route = Route(*args, **kargs)
+        
+        # Apply encoding and errors if its not the defaults and the route 
+        # didn't have one passed in.
+        if (self.encoding != 'utf-8' or self.decode_errors != 'ignore') and \
+           '_encoding' not in kargs:
+            route.encoding = self.encoding
+            route.decode_errors = self.decode_errors
+        
         self.matchlist.append(route)
         if routename:
             self._routenames[routename] = route
@@ -786,7 +849,7 @@ class Mapper(object):
         return None
         
     
-    def generate(self, **kargs):
+    def generate(self, *args, **kargs):
         """Generate a route from a set of keywords
         
         Returns the url text, or None if no URL could be generated.
@@ -811,10 +874,21 @@ class Mapper(object):
         
         controller = kargs.get('controller', None)
         action = kargs.get('action', None)
+
+        # If the URL didn't depend on the SCRIPT_NAME, we'll cache it
+        # keyed by just by kargs; otherwise we need to cache it with
+        # both SCRIPT_NAME and kargs:
+        cache_key = unicode(args).encode('utf8') + unicode(kargs).encode('utf8')
+        if self.environ:
+            cache_key_script_name = '%s:%s' % (
+                self.environ.get('SCRIPT_NAME', ''), cache_key)
+        else:
+            cache_key_script_name = cache_key
         
         # Check the url cache to see if it exists, use it if it does
-        if unicode(kargs) in self.urlcache:
-            return self.urlcache[unicode(kargs)]
+        for key in [cache_key, cache_key_script_name]:
+            if key in self.urlcache:
+                return self.urlcache[key]
         
         actionlist = self._gendict.get(controller) or self._gendict.get('*')
         if not actionlist:
@@ -828,7 +902,9 @@ class Mapper(object):
         cacheset = False
         cachekey = unicode(keys)
         cachelist = sortcache.get(cachekey)
-        if cachelist:
+        if args:
+            keylist = args
+        elif cachelist:
             keylist = cachelist
         else:
             cacheset = True
@@ -896,8 +972,11 @@ class Mapper(object):
                 if self.environ and self.environ.get('SCRIPT_NAME', '') != '' \
                     and not route.absolute:
                     path = self.environ['SCRIPT_NAME'] + path
+                    key = cache_key_script_name
+                else:
+                    key = cache_key
                 if self.urlcache is not None:
-                    self.urlcache[unicode(kargs)] = str(path)
+                    self.urlcache[key] = str(path)
                 return str(path)
             else:
                 continue
@@ -932,7 +1011,7 @@ class Mapper(object):
             Example::
                 
                 map.resource('message', 'messages', collection={'rss':'GET'})
-                # GET /message;rss (maps to the rss action)
+                # GET /message/rss (maps to the rss action)
                 # also adds named route "rss_message"
         
         ``member``
@@ -942,7 +1021,7 @@ class Mapper(object):
             Example::
                 
                 map.resource('message', 'messages', member={'mark':'POST'})
-                # POST /message/1;mark (maps to the mark action)
+                # POST /message/1/mark (maps to the mark action)
                 # also adds named route "mark_message"
         
         ``new``
@@ -952,7 +1031,7 @@ class Mapper(object):
             Example::
                 
                 map.resource('message', 'messages', new={'preview':'POST'})
-                # POST /message/new;preview (maps to the preview action)
+                # POST /message/new/preview (maps to the preview action)
                 # also adds a url named "preview_new_message"
         
         ``path_prefix``
@@ -1005,7 +1084,7 @@ class Mapper(object):
                 >>> url_for('region_location', region_id=13, id=60) 
                 '/regions/13/locations/60'
                 >>> url_for('region_edit_location', region_id=13, id=60) 
-                '/regions/13/locations/60;edit'
+                '/regions/13/locations/60/edit'
 
             Overriding generated ``path_prefix``::
 
@@ -1084,7 +1163,6 @@ class Mapper(object):
         new_path = path + "/new"
         member_path = path + "/:(id)"
         
-        options = {'controller':kwargs.get('controller', controller)}
         options = { 
             'controller': kwargs.get('controller', controller),
             '_member_name': member_name,
@@ -1107,9 +1185,9 @@ class Mapper(object):
             for action in lst:
                 route_options['action'] = action
                 route_name = "%s%s_%s" % (name_prefix, action, collection_name)
-                self.connect(route_name, "%s;%s" % (collection_path, action),
+                self.connect(route_name, "%s/%s" % (collection_path, action),
                                                     **route_options)
-                self.connect("formatted_" + route_name, "%s.:(format);%s" % \
+                self.connect("formatted_" + route_name, "%s/%s.:(format)" % \
                              (collection_path, action), **route_options)
             if primary:
                 route_options['action'] = primary
@@ -1128,7 +1206,7 @@ class Mapper(object):
         for method, lst in new_methods.iteritems():
             route_options = requirements_for(method)
             for action in lst:
-                path = (action == 'new' and new_path) or "%s;%s" % (new_path, 
+                path = (action == 'new' and new_path) or "%s/%s" % (new_path, 
                                                                     action)
                 name = "new_" + member_name
                 if action != 'new':
@@ -1136,11 +1214,11 @@ class Mapper(object):
                 route_options['action'] = action
                 self.connect(name_prefix + name, path, **route_options)
                 path = (action == 'new' and new_path + '.:(format)') or \
-                    "%s.:(format);%s" % (new_path, action)
+                    "%s/%s.:(format)" % (new_path, action)
                 self.connect("formatted_" + name_prefix + name, path, 
                              **route_options)
         
-        requirements_regexp = '[\w\-_]+'
+        requirements_regexp = '[\w\-_\s]+'
 
         # Add the routes that deal with member methods of a resource
         for method, lst in member_methods.iteritems():
@@ -1153,10 +1231,10 @@ class Mapper(object):
             for action in lst:
                 route_options['action'] = action
                 self.connect("%s%s_%s" % (name_prefix, action, member_name),
-                    "%s;%s" % (member_path, action), **route_options)
+                    "%s/%s" % (member_path, action), **route_options)
                 self.connect("formatted_%s%s_%s" % (name_prefix, action, 
                                                     member_name),
-                    "%s.:(format);%s" % (member_path, action), **route_options)
+                    "%s/%s.:(format)" % (member_path, action), **route_options)
             if primary:
                 route_options['action'] = primary
                 self.connect(member_path, **route_options)
