@@ -455,6 +455,14 @@ class TestGeneration(unittest.TestCase):
         assert '/blog/content' == m.generate(controller='content')
         assert '/blog/admin/comments' == m.generate(controller='admin/comments')
 
+        m.environ = dict(SCRIPT_NAME='/notblog')
+
+        assert '/notblog/content/view' == m.generate(controller='content', action='view')
+        assert '/notblog/content' == m.generate(controller='content')
+        assert '/notblog/content' == m.generate(controller='content')
+        assert '/notblog/admin/comments' == m.generate(controller='admin/comments')
+        
+
     def test_url_with_environ_and_absolute(self):
         m = Mapper()
         m.environ = dict(SCRIPT_NAME='/blog')
@@ -493,7 +501,7 @@ class TestGeneration(unittest.TestCase):
         assert baseroute + '.xml' == m.generate(action='index', format='xml', **options)
         assert baseroute + '/new' == m.generate(action='new', **options)
         assert baseroute + '/1' == m.generate(action='show', id='1', **options)
-        assert baseroute + '/1;edit' == m.generate(action='edit',id='1', **options)
+        assert baseroute + '/1/edit' == m.generate(action='edit',id='1', **options)
         assert baseroute + '/1.xml' == m.generate(action='show', id='1',format='xml', **options)
         
         assert baseroute == m.generate(action='create', method='post', **options)
@@ -511,8 +519,8 @@ class TestGeneration(unittest.TestCase):
         assert '/messages/1.xml' == url_for('message', id=1, format='xml')
         assert '/messages/new' == url_for('new_message')
         assert '/messages/1.xml' == url_for('message', id=1, format='xml')
-        assert '/messages/1;edit' == url_for('edit_message', id=1)
-        assert '/messages/1.xml;edit' == url_for('edit_message', id=1, format='xml')
+        assert '/messages/1/edit' == url_for('edit_message', id=1)
+        assert '/messages/1/edit.xml' == url_for('edit_message', id=1, format='xml')
         self._assert_restful_routes(m, options)
     
     def test_resources_with_path_prefix(self):
@@ -528,10 +536,10 @@ class TestGeneration(unittest.TestCase):
         m.create_regs(['messages'])
         options = dict(controller='messages')
         self._assert_restful_routes(m, options)
-        assert '/messages;rss' == m.generate(controller='messages', action='rss')
-        assert '/messages;rss' == url_for('rss_messages')
-        assert '/messages.xml;rss' == m.generate(controller='messages', action='rss', format='xml')
-        assert '/messages.xml;rss' == url_for('formatted_rss_messages', format='xml')
+        assert '/messages/rss' == m.generate(controller='messages', action='rss')
+        assert '/messages/rss' == url_for('rss_messages')
+        assert '/messages/rss.xml' == m.generate(controller='messages', action='rss', format='xml')
+        assert '/messages/rss.xml' == url_for('formatted_rss_messages', format='xml')
     
     def test_resources_with_member_action(self):
         for method in ['put', 'post']:
@@ -540,8 +548,8 @@ class TestGeneration(unittest.TestCase):
             m.create_regs(['messages'])
             options = dict(controller='messages')
             self._assert_restful_routes(m, options)
-            assert '/messages/1;mark' == m.generate(method=method, action='mark', id='1', **options)
-            assert '/messages/1.xml;mark' == m.generate(method=method, action='mark', id='1', format='xml', **options)
+            assert '/messages/1/mark' == m.generate(method=method, action='mark', id='1', **options)
+            assert '/messages/1/mark.xml' == m.generate(method=method, action='mark', id='1', format='xml', **options)
     
     def test_resources_with_new_action(self):
         m = Mapper()
@@ -549,10 +557,10 @@ class TestGeneration(unittest.TestCase):
         m.create_regs(['messages'])
         options = dict(controller='messages')
         self._assert_restful_routes(m, options)
-        assert '/messages/new;preview' == m.generate(controller='messages', action='preview', method='post')
-        assert '/messages/new;preview' == url_for('preview_new_message')
-        assert '/messages/new.xml;preview' == m.generate(controller='messages', action='preview', method='post', format='xml')
-        assert '/messages/new.xml;preview' == url_for('preview_new_message', format='xml')
+        assert '/messages/new/preview' == m.generate(controller='messages', action='preview', method='post')
+        assert '/messages/new/preview' == url_for('preview_new_message')
+        assert '/messages/new/preview.xml' == m.generate(controller='messages', action='preview', method='post', format='xml')
+        assert '/messages/new/preview.xml' == url_for('preview_new_message', format='xml')
     
     def test_resources_with_name_prefix(self):
         m = Mapper()
@@ -560,7 +568,7 @@ class TestGeneration(unittest.TestCase):
         m.create_regs(['messages'])
         options = dict(controller='messages')
         self._assert_restful_routes(m, options)
-        assert '/messages/new;preview' == url_for('category_preview_new_message')
+        assert '/messages/new/preview' == url_for('category_preview_new_message')
         assert None == url_for('category_preview_new_message', method='get')
 
     def test_unicode(self):
@@ -569,7 +577,7 @@ class TestGeneration(unittest.TestCase):
         m = Mapper()
         m.connect(':hoge')
         self.assertEqual("/%s" % hoge_enc, m.generate(hoge=hoge))
-        self.assertTrue(isinstance(m.generate(hoge=hoge), str))
+        self.assert_(isinstance(m.generate(hoge=hoge), str))
 
     def test_unicode_static(self):
         hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
@@ -578,7 +586,7 @@ class TestGeneration(unittest.TestCase):
         m.connect('google-jp', 'http://www.google.co.jp/search', _static=True)
         self.assertEqual("http://www.google.co.jp/search?q=" + hoge_enc,
                          url_for('google-jp', q=hoge))
-        self.assertTrue(isinstance(url_for('google-jp', q=hoge), str))
+        self.assert_(isinstance(url_for('google-jp', q=hoge), str))
 
     def test_other_special_chars(self):
         m = Mapper()
