@@ -14,12 +14,10 @@ class TestRecognition(unittest.TestCase):
         m.connect(':controller/:(action).:(id)')
         m.create_regs(['content'])
         
-        assert None == m.match('/content/view#2')
         assert {'action':'view','controller':'content','id':'2'} == m.match('/content/view.2')
         
         m.connect(':controller/:action/:id')
         m.create_regs(['content', 'find.all'])
-        assert {'action': 'view#2', 'controller': 'content', 'id': None} == m.match('/content/view#2')
         assert {'action':'view','controller':'find.all','id':None} == m.match('/find.all/view')
         assert None == m.match('/findzall/view')
         
@@ -36,11 +34,20 @@ class TestRecognition(unittest.TestCase):
     
     def test_unicode(self):
         hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
-        hoge_enc = urllib.quote_plus(hoge.encode('utf-8'))
+        hoge_enc = hoge.encode('utf-8')
         m = Mapper()
         m.connect(':hoge')
         self.assertEqual({'controller': 'content', 'action': 'index', 'hoge': hoge},
                          m.match('/' + hoge_enc))
+    
+    def test_disabling_unicode(self):
+        hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
+        hoge_enc = urllib.quote_plus(hoge.encode('utf-8'))
+        m = Mapper()
+        m.encoding = None
+        m.connect(':hoge')
+        self.assertEqual({'controller': 'content', 'action': 'index', 'hoge': hoge_enc},
+                         m.match('/' + hoge_enc))
             
     def test_basic_dynamic(self):
         for path in ['hi/:name', 'hi/:(name)']:
@@ -94,7 +101,7 @@ class TestRecognition(unittest.TestCase):
     def test_dynamic_with_default_backwards(self):
         for path in [':action/hi', ':(action)/hi']:
             m = Mapper()
-            m.connect(':action/hi', controller='content')
+            m.connect(path, controller='content')
             m.create_regs([])
 
             self.assertEqual(None, m.match('/'))
@@ -448,6 +455,28 @@ class TestRecognition(unittest.TestCase):
         self.assertEqual({'controller':'content','action':'index', 'id': 'None'}, m.match('/content/index/None.py'))
         self.assertEqual({'controller':'content','action':'list', 'id':'None'}, m.match('/content/list/None.py'))
         self.assertEqual({'controller':'content','action':'show','id':'10'}, m.match('/content/show/10.py'))
+
+    def test_standard_route_with_gaps_and_domains(self):
+        m = Mapper()
+        m.connect('manage/:domain.:ext', controller='admin/user', action='view', ext='html')
+        m.connect(':controller/:action/:id')
+        m.create_regs(['content','admin/user'])
+        
+        self.assertEqual({'controller':'content','action':'index', 'id': 'None.py'}, m.match('/content/index/None.py'))
+        self.assertEqual({'controller':'content','action':'list', 'id':'None.py'}, m.match('/content/list/None.py'))
+        self.assertEqual({'controller':'content','action':'show','id':'10.py'}, m.match('/content/show/10.py'))
+        self.assertEqual({'controller':'content','action':'show.all','id':'10.py'}, m.match('/content/show.all/10.py'))
+        self.assertEqual({'controller':'content','action':'show','id':'www.groovie.org'}, m.match('/content/show/www.groovie.org'))
+        
+        self.assertEqual({'controller':'admin/user','action':'view', 'ext': 'html', 'domain': 'groovie'}, m.match('/manage/groovie'))
+        self.assertEqual({'controller':'admin/user','action':'view', 'ext': 'xml', 'domain': 'groovie'}, m.match('/manage/groovie.xml'))
+    
+    def test_standard_with_domains(self):
+        m = Mapper()
+        m.connect('manage/:domain', controller='domains', action='view')
+        m.create_regs(['domains'])
+        
+        self.assertEqual({'controller':'domains','action':'view','domain':'www.groovie.org'}, m.match('/manage/www.groovie.org'))
     
     def test_default_route(self):
         m = Mapper()
@@ -761,7 +790,7 @@ class TestRecognition(unittest.TestCase):
         
         test_path('/people/2', 'GET')
         assert {'controller':'people', 'action':'show', 'id':'2'} == con.mapper_dict
-        test_path('/people/2;edit', 'GET')
+        test_path('/people/2/edit', 'GET')
         assert {'controller':'people', 'action':'edit', 'id':'2'} == con.mapper_dict
 
         test_path('/people/2', 'DELETE')
@@ -807,11 +836,11 @@ class TestRecognition(unittest.TestCase):
         url = url_for('region_location', region_id=13, id=60)
         assert url == '/regions/13/locations/60'
         
-        test_path('/regions/13/locations/60;edit', 'GET')
+        test_path('/regions/13/locations/60/edit', 'GET')
         assert con.mapper_dict == {'region_id': '13', 'controller': 'locations',
                                    'id': '60', 'action': 'edit'}
         url = url_for('region_edit_location', region_id=13, id=60)
-        assert url == '/regions/13/locations/60;edit'
+        assert url == '/regions/13/locations/60/edit'
         
         test_path('/regions/13/locations/60', 'DELETE')
         assert con.mapper_dict == {'region_id': '13', 'controller': 'locations',
@@ -910,6 +939,22 @@ class TestRecognition(unittest.TestCase):
         self.assertEqual({'controller': 'error', 'action': 'img',
                           'id': 'icon-16.png'}, m.match('/error/img/icon-16.png'))
     
+    def test_various_periods(self):
+        m = Mapper()
+        m.connect('sites/:site/pages/:page')
+        m.create_regs(['content'])
+        
+        self.assertEqual({'action': u'index', 'controller': u'content', 
+                          'site': u'python.com', 'page': u'index.html'}, 
+                         m.match('/sites/python.com/pages/index.html'))
+        m = Mapper()
+        m.connect('sites/:site/pages/:page.:format', format='html')
+        m.create_regs(['content'])
+        
+        self.assertEqual({'action': u'index', 'controller': u'content', 
+                          'site': u'python.com', 'page': u'index', 'format': u'html'}, 
+                         m.match('/sites/python.com/pages/index.html'))
+        
     def test_empty_fails(self):
         m = Mapper()
         m.connect(':controller/:action/:id')
