@@ -23,6 +23,8 @@ class TestUtils(unittest.TestCase):
         
         assert '/blog' == url_for(controller='blog')
         assert '/blog/view/umulat' == url_for(controller='blog', action='view', id=u'umulat')
+        assert '/blog/view/umulat?other=%CE%B1%CF%83%CE%B4%CE%B3' == url_for(controller='blog', action='view', id=u'umulat',
+            other=u'\u03b1\u03c3\u03b4\u03b3')
         
         def raise_url():
             return url_for(u'/some/stirng')
@@ -116,7 +118,27 @@ class TestUtils(unittest.TestCase):
         self.assertEqual('/building/wilma/port/alljacks', url_for())
         self.assertEqual('/', url_for('home'))
         
+    def test_with_route_names_and_hardcode(self):
+        m = self.con.mapper
+        self.con.mapper_dict = {}
+        m.connect('home', '', controller='blog', action='splash')
+        m.connect('category_home', 'category/:section', controller='blog', action='view', section='home')
+        m.connect('building', 'building/:campus/:building/alljacks', controller='building', action='showjacks')
+        m.connect('gallery_thumb', 'gallery/:(img_id)_thumbnail.jpg')
+        m.connect('gallery', 'gallery/:(img_id).jpg')
+        m.create_regs(['content','blog','admin/comments','building'])
 
+        self.con.mapper_dict = dict(controller='building', action='showjacks', campus='wilma', building='port')
+        self.assertEqual('/building/wilma/port/alljacks', url_for())
+        self.assertEqual('/', url_for('home'))
+        self.assertEqual('/gallery/home_thumbnail.jpg', url_for('gallery_thumb', img_id='home'))
+        self.assertEqual('/gallery/home_thumbnail.jpg', url_for('gallery', img_id='home'))
+        
+        m.hardcode_names = True
+        self.assertEqual('/gallery/home_thumbnail.jpg', url_for('gallery_thumb', img_id='home'))
+        self.assertEqual('/gallery/home.jpg', url_for('gallery', img_id='home'))
+        m.hardcode_names = False
+    
     def test_redirect_to(self):
         m = self.con.mapper
         self.con.mapper_dict = {}
@@ -559,17 +581,17 @@ class TestUtilsWithExplicit(unittest.TestCase):
         self.assertEqual(None, url_for(controller='content'))
         self.assertEqual(None, url_for(controller='admin/comments'))
         self.assertEqual('/messages', url_for('messages'))
-        self.assertEqual('/messages;rss', url_for('rss_messages'))
+        self.assertEqual('/messages/rss', url_for('rss_messages'))
         self.assertEqual('/messages/4', url_for('message', id=4))
-        self.assertEqual('/messages/4;edit', url_for('edit_message', id=4))
-        self.assertEqual('/messages/4;mark', url_for('mark_message', id=4))
+        self.assertEqual('/messages/4/edit', url_for('edit_message', id=4))
+        self.assertEqual('/messages/4/mark', url_for('mark_message', id=4))
         self.assertEqual('/messages/new', url_for('new_message'))
         
         self.assertEqual('/messages.xml', url_for('formatted_messages', format='xml'))
-        self.assertEqual('/messages.xml;rss', url_for('formatted_rss_messages', format='xml'))
+        self.assertEqual('/messages/rss.xml', url_for('formatted_rss_messages', format='xml'))
         self.assertEqual('/messages/4.xml', url_for('formatted_message', id=4, format='xml'))
-        self.assertEqual('/messages/4.xml;edit', url_for('formatted_edit_message', id=4, format='xml'))
-        self.assertEqual('/messages/4.xml;mark', url_for('formatted_mark_message', id=4, format='xml'))
+        self.assertEqual('/messages/4/edit.xml', url_for('formatted_edit_message', id=4, format='xml'))
+        self.assertEqual('/messages/4/mark.xml', url_for('formatted_mark_message', id=4, format='xml'))
         self.assertEqual('/messages/new.xml', url_for('formatted_new_message', format='xml'))
         
 
